#!/usr/bin/env python3

def valid_set(digits): return ''.join(sorted(digits)) == '123456789'

from sys import argv, stdin, exit

assert len(argv)==3 and argv[1] in ['easy','hard'], 'correct usage: {} {{easy,hard}} contestants_output'.format(argv[0])

last_in_hard = 8591264

offsets_to_rows = {}
for line in open('pi-250M-substr.txt','r').readlines():
    offset, row = line.split()
    offset = int(offset)
    offsets_to_rows[offset] = row

try:
    try:
        their_output_tokens = open(argv[2],'r').read().split()
    except:
        assert False, "The output must contain 9 rows and 9 offsets, nothing else."

    assert len(their_output_tokens)==18, "The output must contain 9 rows and 9 offsets."
    
    A = [ their_output_tokens[2*n] for n in range(9) ]
    for row in A:
        assert len(row)==9, "Not a valid sudoku (each row must have 9 characters)."
        for d in row:
            assert '1' <= d <= '9', "Not a valid sudoku (each digit must be 1-9)."

    for r in range(9):
        digits = [ A[r][c] for c in range(9) ]
        assert valid_set(digits), "Not a valid sudoku (some row has a duplicate)."

    for c in range(9):
        digits = [ A[r][c] for r in range(9) ]
        assert valid_set(digits), "Not a valid sudoku (some column has a duplicate)."

    for br in range(3):
        for bc in range(3):
            digits = [ A[3*br+r][3*bc+c] for r in range(3) for c in range(3) ]
            assert valid_set(digits), "Not a valid sudoku (some 3x3 block has a duplicate)."

    # if we got here, we have a valid sudoku, we can check whether the reported offsets are ok

    for n in range(9):
        try:
            offset = int( their_output_tokens[2*n+1] )
        except:
            assert False, "Offsets must be positive integers."
        assert offset <= 250000000, "Offsets must not exceed 250,000,000."

        assert offset in offsets_to_rows, "Invalid offset reported (no permutation of 1-9 there)."
    
        assert offsets_to_rows[offset] == their_output_tokens[2*n], "Invalid offset reported (doesn't match sudoku row)."

    for a in range(9):
        for b in range(a):
            oa = int( their_output_tokens[2*a+1] )
            ob = int( their_output_tokens[2*b+1] )
            assert abs(oa-ob) >= 9, "Two of the offsets are too close (permutations overlap)."

    # if we got here, we have a valid sudoku with valid offsets
    
    if argv[1]=='hard': 
        for n in range(9):
            offset = int( their_output_tokens[2*n+1] )
            assert offset <= last_in_hard, "Solution not optimal, offset too large."

except AssertionError as error_message:
    print(error_message)
    exit(1)

print("OK")
